<?php

namespace Drupal\murmurations\EventSubscriber;

use Drupal\Core\Routing\RouteSubscriberBase;
use Symfony\Component\Routing\RouteCollection;
use Symfony\Component\Routing\Route;
use Drupal\murmurations\MurmurationsPluginManager;

/**
 * Because the transaction collection is also the field ui base route, and
 * because views provides a superior listing to the entity's official
 * list_builder, this alters that view's route to comply with the entity.
 */
class RouteSubscriber extends RouteSubscriberBase {

  protected $pluginManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(MurmurationsPluginManager $murmurations_plugin_manager) {
    $this->pluginManager = $murmurations_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection) {

    $plugins= $this->pluginManager->allInstances();
    foreach($plugins as $id => $plugin) {
      if ($plugin->needed()) {
        continue;
      }
      $profile_path = $plugin->getPluginDefinition()['profile_path'];
      // One route to display the json profiles for each plugin
      $route = new Route(
        $profile_path,
        ['_controller' => 'Drupal\murmurations\Page::publishJson'],
        ['_access' =>  'TRUE', '_format' => 'json'],
        ['plugin' => $id]
      );
      if (preg_match('/\{([a-z]+)\}/', $profile_path, $matches)) {
        // enable the autoload
        $entity_type = $matches[1];
        $route->setOption(
          'parameters',
          [$entity_type => ['type' => 'entity:'.$entity_type]]
        );
      }
      $collection->add("murmurations.$id", $route);

      // Two routes for the map and list filter pages for each plugin
      $collection->add(
        "murmurations.$id.search.map",
        new Route(
          "murmurations/$id",
          [
            '_controller' => 'Drupal\murmurations\Page::searchMap',
            '_title_callback' => '\Drupal\murmurations\Page::filterPageTitle'
          ],
          ['_permission' =>  'filter murmurations content'],
          ['plugin' => $id]
        )
      );
      $collection->add(
        "murmurations.$id.search.list",
        new Route(
          "murmurations/$id/list",
          [
            '_controller' => 'Drupal\murmurations\Page::searchList',
            '_title_callback' => '\Drupal\murmurations\Page::filterPageTitle'
          ],
          ['_permission' =>  'filter murmurations content'],
          ['plugin' => $id]
        )
      );
    }
  }
}
