<?php

namespace Drupal\murmurations;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

abstract class MurmurationsPluginBase extends PluginBase implements ContainerFactoryPluginInterface, MurmurationsPluginInterface {

  protected $configFactory;

  function __construct($configuration, $plugin_id, $plugin_definition, $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritDoc}
   */
  function configForm() : array {
    return [];
  }

  /**
   * {@inheritDoc}
   */
  function isReady() : bool {
     return TRUE;
  }

  /**
   * {@inheritDoc}
   */
  public function setObject() : self {
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration($config) {
    $this->configuration = $config + $this->defaultConfiguration();
  }

  /**
   * Gets default configuration for this plugin.
   *
   * @return array
   *   An associative array with the default configuration.
   */
  public function defaultConfiguration() {
    return [
      'displacement' => 3
    ];
  }

  /**
   * {@inheritDoc}
   */
  function needed() : string {
    return '';
  }

  /**
   * {@inheritDoc}
   */
  public function getPublishPath() : string {
    return $this->getPluginDefinition()['profile_path'];
  }

  public function getProfilePaths() : array {
    return [$plugin->getPublishPath()];
  }


  function getProfile() : array {
    global $base_url;
    $fields = [
      'linked_schemas' => [$this->getPluginDefinition()['schema']],
      'geolocation' => $this->configFactory->get('murmurations.settings')->get('fallback_point'),
      'profile_url' => $base_url .'/'. $this->getPublishPath(),
      'primary_url' => $base_url
    ];
    $settings = \Drupal::config('murmurations.settings');
    if ($locality = $settings->get('locality')) {
      $fields['locality'] = $locality;
    }
    if ($region = $settings->get('region')) {
      $fields['region'] = $region;
    }
    $fields['country'] = \Drupal::config('system.date')->get('country.default');
    return $fields;
  }


  /**
   * {@inheritDoc}
   *
   * @note This should always be overridden
   */
  function renderResult(\stdClass $result) : \Drupal\Core\Render\Markup {
    $m = $result->primary_url .'<br />'. implode(',', array_filter([$result->locality, $result->region, $result->country]));
    return \Drupal\Core\Render\Markup::create($m);
  }
}
