<?php

namespace Drupal\murmurations;

use Drupal\Core\Entity\EntityInterface;
use Drupal\murmurations\MurmurationsPluginBase;

abstract class MurmurationsPluginMultipleBase extends MurmurationsPluginBase implements MurmurationsPluginMultipleInterface {

  protected $entity;

  /**
   * {@inheritDoc}
   */
  function needed() : string {
    return $this->getEntityIds() ? '' : t('No itesm are eligible to be published');
  }

  /**
   * {@inheritDoc}
   */
  function setEntity(EntityInterface $entity) : self {
    $this->entity = $entity;
    return $this;
  }

  /**
   * {@inheritDoc}
   */
  function getPublishPath() : string {
    // Replace the parameter in the publish path
    return preg_replace('/\{[a-z]+\}/', $this->entity->id(), $this->getPluginDefinition()['profile_path']);
  }

  public function getProfilePaths() : array {
    foreach ($this->getEntityIds() as $id) {
      $entity_type_id = $this->getPluginDefinition()['entity_type'];
      $entity = \Drupal::entityTypeManager()->getStorage($entity_type_id)->load($id);
      $paths[] = $this->setEntity($entity)->getPublishPath();
    }
  }

  /**
   * {@inheritDoc}
   */
  abstract function getEntityIds() : array;

  function getProfile() :array {
    global $base_url;
    return [
      'geolocation' => murmurations_geolocate($this->entity, $this->configuration['displacement']),
      'profile_url' => $base_url .'/'. $this->getPublishPath(),
      'primary_url' => $this->entity->toUrl()->setAbsolute()->toString()
    ] + parent::getProfile();
  }

}
